#!/bin/sh
SRMD_DIR=/var/srmd_data/
SRMD_ARCHIVE_DIR=/var/srmd_archives/


mkdir -p $SRMD_ARCHIVE_DIR
#Check whether there is old data present
if [ -d "$SRMD_DIR" ]
then
	touch $SRMD_DIR/archived_srmd
	DATE=`date +"%F-%H.%M"`
	#Archive the directory
	tar zcf $SRMD_ARCHIVE_DIR/srmd_archive_$DATE.tgz $SRMD_DIR &> /dev/null
	sync
	#Clean up 
	rm -rf $SRMD_DIR/*
else
	mkdir -p $SRMD_DIR
fi

cd $SRMD_DIR

#Create the 5 seconds bucket. 12 X 5 seconds interval make 1 minute
SEC="0 1";
RSEC="1 0";
for i in ${SEC};
do
	>  srmd_sec_$i
done

#Create the 1 minute bucket. 60 minutes make one hour
MINUTE="0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 57 58 59";
RMINUTE="59 58 57 56 55 54 53 52 51 50 49 48 47 46 45 44 43 42 41 40 39 38 37 36 35 34 33 32 31 30 29 28 27 26 25 24 23 22 21 20 19 18 17 16 15 14 13 12 11 10 9 8 7 6 5 4 3 2 1 0";
for i in ${MINUTE};
do
	> srmd_minute_$i
done

#Create the 1 hour bucket. 24 hours make a day
HOUR="0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23";
RHOUR="23 22 21 20 19 18 17 16 15 14 13 12 11 10 9 8 7 6 5 4 3 2 1 0";
for i in ${HOUR};
do
	> srmd_hour_$i
done

#Create the day bucket. ~31 days make a month
DAY="0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30";
RDAY="30 29 28 27 26 25 24 23 22 21 20 19 18 17 16 15 14 13 12 11 10 9 8 7 6 5 4 3 2 1 0";
for i in ${DAY};
do
	> srmd_day_$i
done

#Create the month bucket. 12 months make an year
MONTH="0 1 2 3 4 5 6 7 8 9 10 11"
RMONTH="11 10 9 8 7 6 5 4 3 2 1 0";
for i in ${MONTH};
do
	> srmd_month_$i
done

chmod a+r *
echo 1 > second_marker
echo 59 > minute_marker
echo 23 > hour_marker
echo 30 > day_marker
echo 11 > month_marker

#Find the number of cores
n_cores=`grep processor /proc/cpuinfo | wc -l | awk '{printf("%d", $1);}'`
while true
do
	#Work on the month bucket
	for month in ${RMONTH};
	do
		> month_join
		#Work on the day bucket
		for day in ${RDAY};
		do
			> day_join
			#Work on the hour bucket
			for hour in ${RHOUR};
			do 
				> hour_join
				#Work on the minute buckets
				for minute in ${RMINUTE};
				do	
					> minute_join
					#First lets work on the 5 second bucket. 
					for i in ${RSEC}; #prev: seq 11 -1 0
					do
						cat minute_join > temp_file
                        /usr/bin/vmstat 1 3 | /usr/bin/tail -n +5 | /bin/awk \
                            '{printf "%07d^TOTAL_CPU\t%d#0\n%07d^TOTAL_IOW\t%d#0\n%07d^TOTAL_SYS\t%d#0\n%07d^TOTAL_USR\t%d#0\n", \
                            0,100-$15, 0,$16, 0,$14, 0, $13}' > srmd_sec_$i
						pidstat -h | tail -n +4 |awk '{printf("%07d^%s\t%0.2f#%d\n", $2, $8, $6i, $7);}' >> srmd_sec_$i
						echo $i > second_marker
						#Add a column to the minute file also.
						join -a 2 temp_file srmd_sec_$i > minute_join 2> /dev/null
						sleep 28 #prev: sleep 5
					done #(1 -1 0)
					#Join the 2 buckets to form a minute of data and keep it.
					awk -v ncores=$n_cores '{ 
						for(i=0;i<ncores;++i) 
							core_count[i] = 0; 
						x = 0; 
						for(i=2;i<4;++i){ 
							split($i,core,"#"); 
							core_count[core[2]]+=1; 
							x+=$i; 
						} 
						printf("%s %.2f", $1, x/2.0); 
						for(i=0;i<ncores;++i) 
							printf ("#%.4f", core_count[i]/2.0); 
						printf("\n");}' minute_join > srmd_minute_$minute
					echo $minute > minute_marker
					cat hour_join > temp_hour
					#Add a column every minute into the hour file. For easier adding at the end.
					join -a 2 temp_hour srmd_minute_$minute > hour_join 2> /dev/null
				done #(60 -1 1)
				awk -v ncores=$n_cores '
				{
					for(i=0;i<ncores;++i)
						core_sum[i] = 0;
					x=0;
					for(i=2;i<62;++i) {
						split($i,core,"#");
						for(j=0;j<ncores;++j)
							core_sum[j]+=core[2+j];
						x+=$i;
					}
					printf ("%s %.2f", $1, x/60.0);
					for(i=0;i<ncores;++i)
						printf("#%.4f", core_sum[i]/60.0);
					printf ("\n");
				}' hour_join > srmd_hour_$hour
				echo $hour > hour_marker
				#Add a column every hour in the day buckect
				cat day_join > temp_day
				join -a 2 temp_day srmd_hour_$hour > day_join 2> /dev/null
			done #(23 -1 0)
			awk -v ncores=$n_cores '
			{
				for(i=0;i<ncores;++i)
					core_sum[i] = 0;
				x=0;
				for(i=2;i<26;++i) {
					split($i,core,"#");
					for(j=0;j<ncores;++j)
						core_sum[j]+=core[2+j];
					x+=$i;
				}
				printf ("%s %.2f", $1, x/24.0);
				for(i=0;i<ncores;++i)
					printf("#%.4f", core_sum[i]/24.0);
				printf ("\n");
			}' day_join > srmd_day_$day
			echo $day > day_marker
			cat month_join > temp_month
			join -a 2 temp_month srmd_day_$day > month_join 2> /dev/null
		done #(31 -1 0)	
		awk -v ncores=$n_cores '
		{
			for(i=0;i<ncores;++i)
				core_sum[i] = 0;
			x=0;
			for(i=2;i<33;++i) {
				split($i,core,"#");
				for(j=0;j<ncores;++j)
					core_sum[j]+=core[2+j];
				x+=$i;
			}
			printf ("%s %.2f", $1, x/31.0);
			for(i=0;i<ncores;++i)
				printf("#%.4f", core_sum[i]/31.0);
			printf ("\n");
		} ' month_join > srmd_month_$month
	done #(11 -1 0)	
done #while	
